function Pvalue=myfisher22(x)
%P=MYFISHER22(X)- Fisher's Exact Probability Test.
%Fisher's exact test of 2x2 contingency tables permits calculation of
%precise probabilities in situation where, as a consequence of small cell
%frequencies, the much more rapid normal approximation and chi-square
%calculations are liable to be inaccurate. The Fisher's exact test involves
%the computations of several factorials to obtain the probability of the
%observed and each of the more extreme tables. Factorials growth quickly,
%so it's necessary use logarithms of factorials. This computations is very
%easy in Matlab because x!=gamma(x+1) and log(x!)=gammaln(x+1). This
%function is now fully vectorized to speed up the computation.
%The routine coumputes the Power and, if necessary, the sample sizes needed
%to achieve a power=0.80 using a modified asymptotic normal method with
%continuity correction as described by Hardeo Sahai and Anwer Khurshid in
%Statistics in Medicine, 1996, Vol. 15, Issue 1: 1-21.
%
% Syntax: 	myfisher22(x)
%      
%     Inputs:
%           X - 2x2 data matrix 
%     Outputs:
%           - Three p-values
%           - Power and sample sizes
%   Example:
%                                     Sex
%                              Male         Female
%                            ---------------------
%                  Recovered     3            6
%            Response                 
%                  Deceased      8            2
%                            ---------------------
%                                       
%
%   x=[3 6; 8 2];
%
%   Calling on Matlab the function: 
%             myfisher22(x)
%
%   Answer is:
%
% 2x2 matrix Fisher's exact test: 9 tables were evaluated
% ----------------------------------------------
%                    P-value  
% ----------------------------------------------
%   Left tail       Right tail         2-tails 
%   (negative)      (positive)         (both)  
% ----------------------------------------------
% 0.0548940224     0.9951178852     0.0697785187
% ----------------------------------------------
%  
% alpha = 0.0500  n1 = 9  n2 = 10
% ----------------------------------------------
%            1-tail          2-tails  
% Z1-b       0.4636          0.1093
% Power      0.6786          0.5435
%  
% To achieve a recommended Power=0.80
% n1          16              19
% n2          18              21
% ---------------------------------------------
%
%           Created by Giuseppe Cardillo
%           giuseppe.cardillo-edta@poste.it
%
% To cite this file, this would be an appropriate format:
% Cardillo G. (2007) MyFisher22: a very compact routine for Fisher's exact
% test on 2x2 matrix
% http://www.mathworks.com/matlabcentral/fileexchange/15434

%Input Error handling
%check if x is a 2x2 matrix
if ~isequal(size(x),[2 2])
    error('Input matrix must be a 2x2 matrix')
end
if ~all(isfinite(x(:))) || ~all(isnumeric(x(:)))
    error('Warning: all X values must be numeric and finite')
end
if ~isequal(x(:),round(x(:)))
    error('Warning: X data matrix values must be whole numbers')
end

Rs=sum(x,2); %rows sum
Cs=sum(x); %columns sum
N=sum(Rs); %sum of all elements

%Rearrange the matrix if necessary.
if ~issorted(Rs)
    x=flipud(x);
    Rs=sort(Rs);
end
if ~issorted(Cs)
    x=fliplr(x);
    Cs=sort(Cs);
end

%           X - 2x2 data matrix composed like this: 
%          ___________
%         |  A  |  B  |Rs1
%         |_____|_____|
%         |  C  |  D  |Rs2
%         |_____|_____|____
%           Cs1  Cs2    N
%recall that Fisher's P=[Rs(1)!Rs(2)!Cs(1)!Cs(2)!]/[N!A!B!C!D!]
%Log(A*B)=Log(A)+Log(B) and Log(A/B)=Log(A)-Log(B)

%A 2x2 matrix has only 1 degree of freedom...
A=0:1:min(Rs(1),Cs(1)); %all possible values for the first cell
z=[A;Rs(2)-Cs(1)+A;Rs(1)-A;Cs(1)-A;]; %all possible matrices

%          The first table will be: 
%          _____________
%         |  0  | Rs1   |Rs1
%         |_____|_______|
%         | Cs1 |Cs2-Rs1|Rs2
%         |_____|_______|____
%           Cs1  Cs2      N
%
%P0=[Rs(1)!Rs(2)!Cs(1)!Cs(2)!]/[N!0!Rs(1)!Cs(1)!D!]=[Rs(2)!Cs(2)!]/[N!D!]
np=zeros(1,length(A)); %p-values vector preallocation
np(1)=exp(sum(gammaln([Rs(2) Cs(1)]+[1 1])-gammaln([N z(2)]+[1 1])));
%remember that np(i+1)=np(i)*[B(i)*C(i)]/(A(i+1)*D(i+1)]=np(i)*f(i)
%and this is vectorizable!
f=prod(z(3:4,1:end-1))./prod(z(1:2,2:end));
np(2:end)=np(1).*cumprod(f);

%now compute the 1-tailed p-values and the 2-tailed p-value
W=x(1)+1;
if x(1)<=round(A(end)/2) %choose direction
    P=[sum(np(1:W)) sum(np(W:end)) sum(np(np<=np(W)))];
else
    P=[sum(np(W:end)) sum(np(1:W)) sum(np(np<=np(W)))];
end

%display results
tr=repmat('-',1,55); %Set up the divisor
disp(' ')
fprintf('2x2 matrix Fisher''s exact test: %0.0f tables were evaluated\n',size(z,2))
disp(tr)
fprintf('\t\t\t\t\t P-value\n'); 
disp(tr)
fprintf('Left tail\t\t\tRight tail\t\t\t2-tails\n'); 
fprintf('(negative)\t\t\t(positive)\t\t\t(both)\n'); 
disp(tr)
fprintf('%0.10f\t\t%0.10f\t\t%0.10f\n',P);
disp(tr)
disp(' ')

%power (Asymptotic normal method)
Za=-realsqrt(2).*erfcinv(2.*[0.95 0.975]);
p=x(:,1)./Rs;
d=abs(diff(p));
k=Rs(2)/Rs(1);
q=1-p;
pm=(p(1)+k*p(2))/(k+1);
qm=1-pm;
Z1_b=abs((realsqrt(Rs(1)*d^2)-Za.*realsqrt((1+1/k)*pm*qm))/realsqrt(p(1)*q(1)+p(2)*q(2)/k));
pwr=0.5.*erfc(-Z1_b./realsqrt(2));
fprintf('alpha = %0.4f  n1 = %d  n2 = %d\n',0.05,Rs)
disp(tr)
fprintf('\t\t\t\t1-tail\t\t\t2-tails\n'); 
fprintf('Z1-b\t\t\t%0.4f\t\t\t%0.4f\n',Z1_b)
fprintf('Power\t\t\t%0.4f\t\t\t%0.4f\n',pwr)
if any(pwr<0.8)
    %sample size (Modified Asymptotic normal method with continuity correction)
    nstar=(Za.*realsqrt(pm*qm*(1+1/k))+(-realsqrt(2)*erfcinv(1.6))*realsqrt(p(1)*q(1)+p(2)*q(2)/k)).^2./d^2;
    n1=round(nstar./4.*(1+realsqrt(1+2*(k+1)./(k*d.*nstar))).^2);
    n2=round(k.*n1);
    disp(' ')
    disp('To achieve a recommended Power=0.80')
    if n1(1)<1000
        fprintf('n1\t\t\t\t%d\t\t\t\t%d\n',n1)
    else
        fprintf('n1\t\t\t\t%d\t\t\t%d\n',n1)
    end
    if n2(1)<1000
        fprintf('n2\t\t\t\t%d\t\t\t\t%d\n',n2)
    else
        fprintf('n2\t\t\t\t%d\t\t\t%d\n',n2)
    end
end
disp(tr)
if nargout
    Pvalue=P;
end